#ifndef __TG3_COMPAT_H__
#define __TG3_COMPAT_H__

/* NOTE: This compat file was (relunctantly) added because an update to the tg3
 * driver was required as a Driver Update Program (DUP) and so adding 
 * functionality to the 5.8 PCI driver would not allow for this driver to work
 * in previous versions. A bugzilla will be created to document this.
 */

#define PCI_VPD_MAX_POLL	40
#define PCI_VPD_LEN		(1 + PCI_VPD_ADDR_MASK)

static inline ssize_t pci_read_vpd(struct pci_dev *dev,
				   loff_t pos, size_t count, void *arg)
{
	loff_t	end = pos + count;
	u8	*buf = arg;
	u16	v16;
	u32	v32;
	int	attempts = PCI_VPD_MAX_POLL;
	int	ret;

	static struct cache {
		struct pci_dev *dev;
		u8      cap;
	} cache = {0,0};

	if (pos < 0 || pos > PCI_VPD_LEN || end > PCI_VPD_LEN)
		return -EINVAL;

	if (cache.dev != dev) {
		cache.cap = pci_find_capability(dev, PCI_CAP_ID_VPD);
		cache.dev = dev;
	}

	while (pos < end) {
		unsigned int i, skip;

		ret = pci_write_config_word(dev, cache.cap + PCI_VPD_ADDR,
					    pos & ~3);
		if (ret < 0)
			return ret;

		attempts = PCI_VPD_MAX_POLL;
		do {
			udelay(10);
			pci_read_config_word(dev, cache.cap + PCI_VPD_ADDR, &v16);
		} while (!(v16 & PCI_VPD_ADDR_F) && --attempts);

		if (attempts <= 0)
			return -ETIMEDOUT;

		ret = pci_read_config_dword(dev, cache.cap + PCI_VPD_DATA, &v32);
		if (ret < 0)
			break;

		skip = pos & 3;
		for (i = 0;  i < sizeof (u32); ++i) {
			if (i >= skip) {
				*buf++ = v32;
				if (++pos == end)
					break;
			}
		v32 >>= 8;
		}
	}

	return count;
}

#endif
