/*
 * Copyright (c) 2005-2012 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * Linux driver for Brocade Fibre Channel Host Bus Adapter.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License (GPL) Version 2 as
 * published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */
#ifndef BFAD_IOCTL_H
#define BFAD_IOCTL_H

#include <linux/ioctl.h>

#include "bfa_defs.h"
#include "bfa_defs_fcs.h"

/* Definitions of vendor unique structures and command codes */
enum {
	BFAD_IOC_ENABLE = 0x3,
	BFAD_IOC_DISABLE,
	BFAD_IOC_GET_ATTR = 0x6,
	BFAD_IOC_GET_INFO,
	BFAD_IOC_GET_STATS,
	BFAD_IOC_RESET_STATS,
	BFAD_IOC_SET_ADAPTER_NAME,
	BFAD_IOC_SET_PORT_NAME,
	BFAD_IOC_GET_FWSTATS,
	BFAD_IOC_RESET_FWSTATS,
	BFAD_IOC_PCIFN_CFG,
	BFAD_ADAPTER_CFG_MODE,
	BFAD_DIAG_TEMP = 0x13,
	BFAD_DIAG_MEMTEST = 0x15,
	BFAD_DIAG_LOOPBACK,
	BFAD_DIAG_FWPING,
	BFAD_DIAG_QUEUETEST,
	BFAD_DIAG_SFP = 0x1a,
	BFAD_DIAG_LED,
	BFAD_DIAG_BEACON_LPORT,
	BFAD_DIAG_LB_STAT = 0x1e,
	BFAD_DEBUG_FW_STATE_CLR = 0x23,
	BFAD_DEBUG_FW_CORE,
	BFAD_DEBUG_PORTLOG,
	BFAD_DEBUG_PORTLOG_CLR,
	BFAD_DEBUG_PORTLOG_CTL,
	BFAD_DEBUG_START_DTRC,
	BFAD_DEBUG_STOP_DTRC,
	BFAD_PORT_ENABLE = 0x2e,
	BFAD_PORT_DISABLE,
	BFAD_PORT_GET_ATTR,
	BFAD_PORT_GET_STATS,
	BFAD_PORT_RESET_STATS = 0x34,
	BFAD_PORT_CFG_TOPO,
	BFAD_PORT_CFG_SPEED,
	BFAD_PORT_CFG_ALPA,
	BFAD_PORT_CLR_ALPA,
	BFAD_PORT_CFG_MAXFRSZ,
	BFAD_PORT_CFG_MODE = 0x3e,
	BFAD_PORT_BBSC_ENABLE,
	BFAD_PORT_BBSC_DISABLE,
	BFAD_CEE_RESET_STATS,
	BFAD_CEE_GET_ATTR,
	BFAD_CEE_GET_STATS,
	BFAD_SFP_MEDIA = 0x45,
	BFAD_SFP_SPEED,
	BFAD_ETHBOOT_CFG,
	BFAD_ETHBOOT_QUERY,
	BFAD_FLASH_GET_ATTR,
	BFAD_FLASH_ERASE_PART,
	BFAD_FLASH_UPDATE_PART,
	BFAD_FLASH_READ_PART,
	BFAD_FLASH_ENABLE_OPTROM = 0x4e,
	BFAD_FLASH_DISABLE_OPTROM,
	BFAD_PHY_UPDATE_FW = 0x51,
	BFAD_PHY_READ_FW,
	BFAD_PHY_GET_ATTR,
	BFAD_PHY_GET_STATS,
	BFAD_PCIFN_CREATE,
	BFAD_PCIFN_DELETE,
	BFAD_PCIFN_BW = 0x57,
	BFAD_VHBA_QUERY = 0x5d,
	BFAD_VHBA_ENABLE,
	BFAD_VHBA_DISABLE,
	BFAD_IOCFC_GET_ATTR = 0x01,
	BFAD_IOCFC_SET_INTR,
	BFAD_LPORT_GET_ATTR = 0x01,
	BFAD_LPORT_GET_IOSTATS,
	BFAD_LPORT_GET_STATS,
	BFAD_LPORT_RESET_STATS,
	BFAD_LPORT_GET_RPORTS,
	BFAD_RPORT_GET_STATS = 0x03,
	BFAD_RPORT_RESET_STATS,
	BFAD_RPORT_GET_ATTR,
	BFAD_RPORT_GET_ADDR,
	BFAD_RPORT_SET_SPEED,
	BFAD_VPORT_GET_ATTR = 0x05,
	BFAD_VPORT_GET_STATS,
	BFAD_VPORT_RESET_STATS,
	BFAD_VF_GET_STATS = 0x08,
	BFAD_VF_RESET_STATS,
	BFAD_QOS_ENABLE = 0x01,
	BFAD_QOS_DISABLE,
	BFAD_QOS_GET_ATTR,
	BFAD_QOS_GET_VC_ATTR,
	BFAD_QOS_GET_STATS,
	BFAD_QOS_RESET_STATS,
	BFAD_RATELIM_ENABLE = 0x01,
	BFAD_RATELIM_DISABLE,
	BFAD_RATELIM_DEF_SPEED,
	BFAD_FCPIM_FAILOVER = 0x01,
	BFAD_FCPIM_MODSTATS = 0x03,
	BFAD_FCPIM_MODSTATSCLR,
	BFAD_FCPIM_PROFILE_ON,
	BFAD_FCPIM_PROFILE_OFF,
	BFAD_FCPIM_DEL_ITN_STATS,
	BFAD_BOOT_CFG = 0x01,
	BFAD_BOOT_QUERY,
	BFAD_PREBOOT_QUERY,
	BFAD_ITNIM_GET_ATTR = 0x01,
	BFAD_ITNIM_GET_IOSTATS,
	BFAD_ITNIM_RESET_STATS,
	BFAD_ITNIM_GET_ITNSTATS,
	BFAD_ITNIM_GET_IOPROFILE,
	BFAD_FABRIC_GET_LPORTS = 0x01,
	BFAD_FCPORT_GET_STATS = 0x01,
	BFAD_FCPORT_RESET_STATS,
	BFAD_FCPORT_ENABLE,
	BFAD_FCPORT_DISABLE,
	BFAD_TRUNK_ENABLE = 0x01,
	BFAD_TRUNK_DISABLE,
	BFAD_TRUNK_GET_ATTR,
	BFAD_FAA_QUERY = 0x03,
	BFAD_SEND_FC = 0x02,
};

struct bfa_ioctl_gen_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
};

struct bfa_ioctl_portlogctl_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	bfa_boolean_t	ctl;
	int		inst_no;
};

struct bfa_ioctl_fcpim_profile_s {
	bfa_status_t    status;
	u16		bfad_num;
	u16		rsvd;
};

struct bfa_ioctl_itnim_ioprofile_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		lpwwn;
	wwn_t		rpwwn;
	struct bfa_itnim_ioprofile_s ioprofile;
};

struct bfa_ioctl_fcport_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	union bfa_fcport_stats_u stats;
};

struct bfa_ioctl_ioc_name_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	char		name[BFA_ADAPTER_SYM_NAME_LEN];
};

struct bfa_ioctl_ioc_info_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	char		serialnum[64];
	char		hwpath[BFA_STRING_32];
	char		adapter_hwpath[BFA_STRING_32];
	char		guid[BFA_ADAPTER_SYM_NAME_LEN*2];
	char		name[BFA_ADAPTER_SYM_NAME_LEN];
	char		port_name[BFA_ADAPTER_SYM_NAME_LEN];
	char		eth_name[BFA_ADAPTER_SYM_NAME_LEN];
	wwn_t		pwwn;
	wwn_t		nwwn;
	wwn_t		factorypwwn;
	wwn_t		factorynwwn;
	mac_t		mac;
	mac_t		factory_mac; /* Factory mac address */
	mac_t		current_mac; /* Currently assigned mac address */
	enum bfa_ioc_type_e	ioc_type;
	u16		pvid; /* Port vlan id */
	u16		rsvd1;
	u32		host;
	u32		bandwidth; /* For PF support */
	u32		rsvd2;
};

struct bfa_ioctl_ioc_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_ioc_attr_s  ioc_attr;
};

struct bfa_ioctl_ioc_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_ioc_stats_s ioc_stats;
};

struct bfa_ioctl_ioc_fwstats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		buf_size;
	u32		rsvd1;
	u64		buf_ptr;
};

struct bfa_ioctl_iocfc_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_iocfc_attr_s	iocfc_attr;
};

struct bfa_ioctl_iocfc_intr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_iocfc_intr_attr_s attr;
};

struct bfa_ioctl_port_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_port_attr_s	attr;
};

struct bfa_ioctl_port_cfg_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		param;
	u32		rsvd1;
};

struct bfa_ioctl_port_cfg_maxfrsize_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		maxfrsize;
};

struct bfa_ioctl_port_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		buf_size;
	u32		rsvd1;
	u64		buf_ptr;
};

struct bfa_ioctl_lport_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	struct bfa_lport_attr_s port_attr;
};

struct bfa_ioctl_lport_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	struct bfa_lport_stats_s port_stats;
};

struct bfa_ioctl_lport_iostats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	struct bfa_itnim_iostats_s iostats;
};

struct bfa_ioctl_lport_get_rports_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	u64		rbuf_ptr;
	u32		nrports;
	u32		rsvd;
};

struct bfa_ioctl_rport_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	wwn_t		rpwwn;
	struct bfa_rport_attr_s attr;
};

struct bfa_ioctl_rport_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	wwn_t		rpwwn;
	struct bfa_rport_stats_s stats;
};

struct bfa_ioctl_rport_scsi_addr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	wwn_t		rpwwn;
	u32		host;
	u32		bus;
	u32		target;
	u32		lun;
};

struct bfa_ioctl_rport_reset_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		pwwn;
	wwn_t		rpwwn;
};

struct bfa_ioctl_rport_set_speed_s {
	bfa_status_t		status;
	u16			bfad_num;
	u16			vf_id;
	enum bfa_port_speed	speed;
	u32			rsvd;
	wwn_t			pwwn;
	wwn_t			rpwwn;
};

struct bfa_ioctl_vport_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		vpwwn;
	struct bfa_vport_attr_s vport_attr;
};

struct bfa_ioctl_vport_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		vpwwn;
	struct bfa_vport_stats_s vport_stats;
};

struct bfa_ioctl_reset_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		vpwwn;
};

struct bfa_ioctl_fabric_get_lports_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	u64		buf_ptr;
	u32		nports;
	u32		rsvd;
};

struct bfa_ioctl_trl_speed_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	enum bfa_port_speed speed;
};

struct bfa_ioctl_fcpim_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		param;
};

struct bfa_ioctl_fcpim_modstats_s {
	bfa_status_t	status;
	u16		bfad_num;
	struct bfa_itnim_iostats_s modstats;
};

struct bfa_ioctl_fcpim_del_itn_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	struct bfa_fcpim_del_itn_stats_s modstats;
};

struct bfa_ioctl_fcpim_modstatsclr_s {
	bfa_status_t	status;
	u16		bfad_num;
};

struct bfa_ioctl_itnim_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		lpwwn;
	wwn_t		rpwwn;
	struct bfa_itnim_attr_s	attr;
};

struct bfa_ioctl_itnim_iostats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		lpwwn;
	wwn_t		rpwwn;
	struct bfa_itnim_iostats_s iostats;
};

struct bfa_ioctl_itnim_itnstats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		lpwwn;
	wwn_t		rpwwn;
	struct bfa_itnim_stats_s itnstats;
};

struct bfa_ioctl_pcifn_cfg_s {
	bfa_status_t		status;
	u16			bfad_num;
	u16			rsvd;
	struct bfa_ablk_cfg_s	pcifn_cfg;
};

struct bfa_ioctl_pcifn_s {
	bfa_status_t		status;
	u16			bfad_num;
	u16			pcifn_id;
	u32			bandwidth;
	u8			port;
	enum bfi_pcifn_class	pcifn_class;
	u8			rsvd[1];
};

struct bfa_ioctl_adapter_cfg_mode_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_adapter_cfg_mode_s	cfg;
};

struct bfa_ioctl_port_cfg_mode_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		instance;
	struct bfa_port_cfg_mode_s cfg;
};

struct bfa_ioctl_faa_attr_s {
	bfa_status_t		status;
	u16			bfad_num;
	u16			rsvd;
	struct bfa_faa_attr_s	faa_attr;
};

struct bfa_ioctl_cee_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		buf_size;
	u32		rsvd1;
	u64		buf_ptr;
};

struct bfa_ioctl_cee_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		buf_size;
	u32		rsvd1;
	u64		buf_ptr;
};

struct bfa_ioctl_sfp_media_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	enum bfa_defs_sfp_media_e media;
};

struct bfa_ioctl_sfp_speed_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	enum bfa_port_speed speed;
};

struct bfa_ioctl_flash_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_flash_attr_s attr;
};

struct bfa_ioctl_flash_s {
	bfa_status_t	status;
	u16		bfad_num;
	u8		instance;
	u8		rsvd;
	enum  bfa_flash_part_type type;
	int		bufsz;
	u64		buf_ptr;
};

struct bfa_ioctl_diag_get_temp_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_diag_results_tempsensor_s result;
};

struct bfa_ioctl_diag_memtest_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd[3];
	u32		pat;
	struct bfa_diag_memtest_result result;
	struct bfa_diag_memtest_s memtest;
};

struct bfa_ioctl_diag_loopback_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	enum bfa_port_opmode opmode;
	enum bfa_port_speed speed;
	u32		lpcnt;
	u32		pat;
	struct bfa_diag_loopback_result_s result;
};

struct bfa_ioctl_diag_fwping_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		cnt;
	u32		pattern;
	struct bfa_diag_results_fwping result;
};

struct bfa_ioctl_diag_qtest_s {
	bfa_status_t	status;
	u16	bfad_num;
	u16	rsvd;
	u32	force;
	u32	queue;
	struct bfa_diag_qtest_result_s result;
};

struct bfa_ioctl_sfp_show_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct sfp_mem_s sfp;
};

struct bfa_ioctl_diag_led_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_diag_ledtest_s ledtest;
};

struct bfa_ioctl_diag_beacon_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	bfa_boolean_t   beacon;
	bfa_boolean_t   link_e2e_beacon;
	u32		second;
};

struct bfa_ioctl_diag_lb_stat_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
};

struct bfa_ioctl_phy_attr_s {
	bfa_status_t	status;
	u16	bfad_num;
	u16	instance;
	struct bfa_phy_attr_s	attr;
};

struct bfa_ioctl_phy_s {
	bfa_status_t	status;
	u16	bfad_num;
	u16	instance;
	u64	bufsz;
	u64	buf_ptr;
};

struct bfa_ioctl_debug_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	u32		bufsz;
	int		inst_no;
	u64		buf_ptr;
	u64		offset;
};

struct bfa_ioctl_phy_stats_s {
	bfa_status_t	status;
	u16	bfad_num;
	u16	instance;
	struct bfa_phy_stats_s	stats;
};

struct bfa_ioctl_vhba_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		pcifn_id;
	struct bfa_vhba_attr_s	attr;
};

struct bfa_ioctl_boot_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_boot_cfg_s	cfg;
};

struct bfa_ioctl_preboot_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_boot_pbc_s	cfg;
};

struct bfa_ioctl_ethboot_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct  bfa_ethboot_cfg_s  cfg;
};

struct bfa_ioctl_trunk_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_trunk_attr_s attr;
};

struct bfa_ioctl_qos_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_qos_attr_s	attr;
};

struct bfa_ioctl_qos_vc_attr_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		rsvd;
	struct bfa_qos_vc_attr_s attr;
};

struct bfa_ioctl_vf_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	struct bfa_vf_stats_s	stats;
};

struct bfa_ioctl_vf_reset_stats_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
};

struct bfa_ioctl_fcpt_s {
	bfa_status_t	status;
	u16		bfad_num;
	u16		vf_id;
	wwn_t		lpwwn;
	wwn_t		dpwwn;
	u32		tsecs;
	int		cts;
	enum fc_cos	cos;
	u32		rsvd;
	struct fchs_s	fchs;
	int		nport_nexus_needed;
	u32		reqsz;
	u32		rspsz;
	u32		rsvd1;
	u64		reqbuf_ptr;
	u64		rspbuf_ptr;
};

#define bfa_ioctl_fcpt_t struct bfa_ioctl_fcpt_s

/* IOCTL cmd code */
#define BFA_MAGIC	0x43
#define IOCFC_MAGIC	0xB0
#define LPORT_MAGIC	0xB1
#define RPORT_MAGIC	0xB2
#define VPORT_MAGIC	0xB3
#define VF_MAGIC	0xB4
#define QOS_MAGIC	0xB6
#define RATELIM_MAGIC	0xB7
#define FCPIM_MAGIC	0xB8
#define BOOT_MAGIC	0xB9
#define PASSTHRU_MAGIC	0xBA
#define ITNIM_MAGIC	0xBB
#define FABRIC_MAGIC	0xBE
#define FCPORT_MAGIC	0xC1
#define TRUNK_MAGIC	0xC2
#define FAA_MAGIC	0xC3

#define IOCMD_IOC_ENABLE \
	_IOWR(BFA_MAGIC, BFAD_IOC_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_IOC_DISABLE \
	_IOWR(BFA_MAGIC, BFAD_IOC_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_IOC_GET_ATTR \
	_IOWR(BFA_MAGIC, BFAD_IOC_GET_ATTR, struct bfa_ioctl_ioc_attr_s)
#define IOCMD_IOC_GET_INFO \
	_IOWR(BFA_MAGIC, BFAD_IOC_GET_INFO, struct bfa_ioctl_ioc_info_s)
#define IOCMD_IOC_GET_STATS \
	_IOWR(BFA_MAGIC, BFAD_IOC_GET_STATS, struct bfa_ioctl_ioc_stats_s)
#define IOCMD_IOC_RESET_STATS \
	_IOWR(BFA_MAGIC, BFAD_IOC_RESET_STATS, struct bfa_ioctl_gen_s)
#define IOCMD_IOC_SET_ADAPTER_NAME \
	_IOWR(BFA_MAGIC, BFAD_IOC_SET_ADAPTER_NAME, struct bfa_ioctl_ioc_name_s)
#define IOCMD_IOC_SET_PORT_NAME \
	_IOWR(BFA_MAGIC, BFAD_IOC_SET_PORT_NAME, struct bfa_ioctl_ioc_name_s)
#define IOCMD_IOC_GET_FWSTATS \
	_IOWR(BFA_MAGIC, BFAD_IOC_GET_FWSTATS, struct bfa_ioctl_ioc_fwstats_s)
#define IOCMD_IOC_RESET_FWSTATS \
	_IOWR(BFA_MAGIC, BFAD_IOC_RESET_FWSTATS, struct bfa_ioctl_gen_s)
#define IOCMD_IOC_PCIFN_CFG \
	_IOWR(BFA_MAGIC, BFAD_IOC_PCIFN_CFG, struct bfa_ioctl_pcifn_cfg_s)
#define IOCMD_ADAPTER_CFG_MODE \
	_IOWR(BFA_MAGIC, BFAD_ADAPTER_CFG_MODE, \
	      struct bfa_ioctl_adapter_cfg_mode_s)
#define IOCMD_DIAG_TEMP \
	_IOWR(BFA_MAGIC, BFAD_DIAG_TEMP, struct bfa_ioctl_diag_get_temp_s)
#define IOCMD_DIAG_MEMTEST \
	_IOWR(BFA_MAGIC, BFAD_DIAG_MEMTEST, struct bfa_ioctl_diag_memtest_s)
#define IOCMD_DIAG_LOOPBACK \
	_IOWR(BFA_MAGIC, BFAD_DIAG_LOOPBACK, struct bfa_ioctl_diag_loopback_s)
#define IOCMD_DIAG_FWPING \
	_IOWR(BFA_MAGIC, BFAD_DIAG_FWPING, struct bfa_ioctl_diag_fwping_s)
#define IOCMD_DIAG_QUEUETEST \
	_IOWR(BFA_MAGIC, BFAD_DIAG_QUEUETEST, struct bfa_ioctl_diag_qtest_s)
#define IOCMD_DIAG_SFP \
	_IOWR(BFA_MAGIC, BFAD_DIAG_SFP, struct bfa_ioctl_sfp_show_s)
#define IOCMD_DIAG_LED \
	_IOWR(BFA_MAGIC, BFAD_DIAG_LED, struct bfa_ioctl_diag_led_s)
#define IOCMD_DIAG_BEACON_LPORT \
	_IOWR(BFA_MAGIC, BFAD_DIAG_BEACON_LPORT, struct bfa_ioctl_diag_beacon_s)
#define IOCMD_DIAG_LB_STAT \
	_IOWR(BFA_MAGIC, BFAD_DIAG_LB_STAT, struct bfa_ioctl_diag_lb_stat_s)
#define IOCMD_SFP_MEDIA \
	_IOWR(BFA_MAGIC, BFAD_SFP_MEDIA, struct bfa_ioctl_sfp_media_s)
#define IOCMD_SFP_SPEED \
	_IOWR(BFA_MAGIC, BFAD_SFP_SPEED, struct bfa_ioctl_sfp_speed_s)
#define IOCMD_DEBUG_FW_STATE_CLR \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_FW_STATE_CLR, struct bfa_ioctl_gen_s)
#define IOCMD_DEBUG_FW_CORE \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_FW_CORE, struct bfa_ioctl_debug_s)
#define IOCMD_DEBUG_PORTLOG \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_PORTLOG, struct bfa_ioctl_debug_s)
#define IOCMD_DEBUG_PORTLOG_CLR \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_PORTLOG_CLR, struct bfa_ioctl_gen_s)
#define IOCMD_DEBUG_PORTLOG_CTL \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_PORTLOG_CTL, struct bfa_ioctl_portlogctl_s)
#define IOCMD_DEBUG_START_DTRC \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_START_DTRC, struct bfa_ioctl_gen_s)
#define IOCMD_DEBUG_STOP_DTRC \
	_IOWR(BFA_MAGIC, BFAD_DEBUG_STOP_DTRC, struct bfa_ioctl_gen_s)
#define IOCMD_PORT_ENABLE \
	_IOWR(BFA_MAGIC, BFAD_PORT_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_PORT_DISABLE \
	_IOWR(BFA_MAGIC, BFAD_PORT_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_PORT_GET_ATTR \
	_IOWR(BFA_MAGIC, BFAD_PORT_GET_ATTR, struct bfa_ioctl_port_attr_s)
#define IOCMD_PORT_GET_STATS \
	_IOWR(BFA_MAGIC, BFAD_PORT_GET_STATS, struct bfa_ioctl_port_stats_s)
#define IOCMD_PORT_RESET_STATS \
	_IOWR(BFA_MAGIC, BFAD_PORT_RESET_STATS, struct bfa_ioctl_gen_s)
#define IOCMD_PORT_CFG_TOPO \
	_IOWR(BFA_MAGIC, BFAD_PORT_CFG_TOPO, struct bfa_ioctl_port_cfg_s)
#define IOCMD_PORT_CFG_SPEED \
	_IOWR(BFA_MAGIC, BFAD_PORT_CFG_SPEED, struct bfa_ioctl_port_cfg_s)
#define IOCMD_PORT_CFG_ALPA \
	_IOWR(BFA_MAGIC, BFAD_PORT_CFG_ALPA, struct bfa_ioctl_port_cfg_s)
#define IOCMD_PORT_CLR_ALPA \
	_IOWR(BFA_MAGIC, BFAD_PORT_CLR_ALPA, struct bfa_ioctl_port_cfg_s)
#define IOCMD_PORT_CFG_MAXFRSZ \
	_IOWR(BFA_MAGIC, BFAD_PORT_CFG_MAXFRSZ, \
	      struct bfa_ioctl_port_cfg_maxfrsize_s)
#define IOCMD_PORT_CFG_MODE \
	_IOWR(BFA_MAGIC, BFAD_PORT_CFG_MODE, struct bfa_ioctl_port_cfg_mode_s)
#define IOCMD_PORT_BBSC_ENABLE \
	_IOWR(BFA_MAGIC, BFAD_PORT_BBSC_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_PORT_BBSC_DISABLE \
	_IOWR(BFA_MAGIC, BFAD_PORT_BBSC_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_CEE_GET_ATTR \
	_IOWR(BFA_MAGIC, BFAD_CEE_GET_ATTR, struct bfa_ioctl_cee_attr_s)
#define IOCMD_CEE_GET_STATS \
	_IOWR(BFA_MAGIC, BFAD_CEE_GET_STATS, struct bfa_ioctl_cee_stats_s)
#define IOCMD_CEE_RESET_STATS \
	_IOWR(BFA_MAGIC, BFAD_CEE_RESET_STATS, struct bfa_ioctl_gen_s)
#define IOCMD_ETHBOOT_CFG \
	_IOWR(BFA_MAGIC, BFAD_ETHBOOT_CFG, struct bfa_ioctl_ethboot_s)
#define IOCMD_ETHBOOT_QUERY \
	_IOWR(BFA_MAGIC, BFAD_ETHBOOT_QUERY, struct bfa_ioctl_ethboot_s)
#define IOCMD_FLASH_GET_ATTR \
	_IOWR(BFA_MAGIC, BFAD_FLASH_GET_ATTR, struct bfa_ioctl_flash_attr_s)
#define IOCMD_FLASH_ERASE_PART \
	_IOWR(BFA_MAGIC, BFAD_FLASH_ERASE_PART, struct bfa_ioctl_flash_s)
#define IOCMD_FLASH_UPDATE_PART \
	_IOWR(BFA_MAGIC, BFAD_FLASH_UPDATE_PART, struct bfa_ioctl_flash_s)
#define IOCMD_FLASH_READ_PART \
	_IOWR(BFA_MAGIC, BFAD_FLASH_READ_PART, struct bfa_ioctl_flash_s)
#define IOCMD_FLASH_ENABLE_OPTROM \
	_IOWR(BFA_MAGIC, BFAD_FLASH_ENABLE_OPTROM, struct bfa_ioctl_gen_s)
#define IOCMD_FLASH_DISABLE_OPTROM \
	_IOWR(BFA_MAGIC, BFAD_FLASH_DISABLE_OPTROM, struct bfa_ioctl_gen_s)
#define IOCMD_PHY_UPDATE_FW \
	_IOWR(BFA_MAGIC, BFAD_PHY_UPDATE_FW, struct bfa_ioctl_phy_s)
#define IOCMD_PHY_READ_FW \
	_IOWR(BFA_MAGIC, BFAD_PHY_READ_FW, struct bfa_ioctl_phy_s)
#define IOCMD_PHY_GET_ATTR \
	_IOWR(BFA_MAGIC, BFAD_PHY_GET_ATTR, struct bfa_ioctl_phy_attr_s)
#define IOCMD_PHY_GET_STATS \
	_IOWR(BFA_MAGIC, BFAD_PHY_GET_STATS, struct bfa_ioctl_phy_stats_s)
#define IOCMD_PCIFN_CREATE \
	_IOWR(BFA_MAGIC, BFAD_PCIFN_CREATE, struct bfa_ioctl_pcifn_s)
#define IOCMD_PCIFN_DELETE \
	_IOWR(BFA_MAGIC, BFAD_PCIFN_DELETE, struct bfa_ioctl_pcifn_s)
#define IOCMD_PCIFN_BW \
	_IOWR(BFA_MAGIC, BFAD_PCIFN_BW, struct bfa_ioctl_pcifn_s)
#define IOCMD_VHBA_QUERY \
	_IOWR(BFA_MAGIC, BFAD_VHBA_QUERY, struct bfa_ioctl_vhba_attr_s)
#define IOCMD_VHBA_ENABLE \
	_IOWR(BFA_MAGIC, BFAD_VHBA_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_VHBA_DISABLE \
	_IOWR(BFA_MAGIC, BFAD_VHBA_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_IOCFC_GET_ATTR \
	_IOWR(IOCFC_MAGIC, BFAD_IOCFC_GET_ATTR, struct bfa_ioctl_iocfc_attr_s)
#define IOCMD_IOCFC_SET_INTR \
	_IOWR(IOCFC_MAGIC, BFAD_IOCFC_SET_INTR, struct bfa_ioctl_iocfc_intr_s)
#define IOCMD_LPORT_GET_ATTR \
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_ATTR, struct bfa_ioctl_lport_attr_s)
#define IOCMD_LPORT_GET_IOSTATS \
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_IOSTATS, \
	      struct bfa_ioctl_lport_iostats_s)
#define IOCMD_LPORT_GET_STATS \
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_STATS, struct bfa_ioctl_lport_stats_s)
#define IOCMD_LPORT_RESET_STATS \
	_IOWR(LPORT_MAGIC, BFAD_LPORT_RESET_STATS, \
	      struct bfa_ioctl_reset_stats_s)
#define IOCMD_LPORT_GET_RPORTS \
	_IOWR(LPORT_MAGIC, BFAD_LPORT_GET_RPORTS, \
	      struct bfa_ioctl_lport_get_rports_s)
#define IOCMD_RPORT_GET_STATS \
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_STATS, struct bfa_ioctl_rport_stats_s)
#define IOCMD_RPORT_RESET_STATS \
	_IOWR(RPORT_MAGIC, BFAD_RPORT_RESET_STATS, \
	      struct bfa_ioctl_rport_reset_stats_s)
#define IOCMD_RPORT_GET_ATTR \
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_ATTR, struct bfa_ioctl_rport_attr_s)
#define IOCMD_RPORT_GET_ADDR \
	_IOWR(RPORT_MAGIC, BFAD_RPORT_GET_ADDR, \
	      struct bfa_ioctl_rport_scsi_addr_s)
#define IOCMD_RPORT_SET_SPEED \
	_IOWR(RPORT_MAGIC, BFAD_RPORT_SET_SPEED, \
	      struct bfa_ioctl_rport_set_speed_s)
#define IOCMD_VPORT_GET_ATTR \
	_IOWR(VPORT_MAGIC, BFAD_VPORT_GET_ATTR, struct bfa_ioctl_vport_attr_s)
#define IOCMD_VPORT_GET_STATS \
	_IOWR(VPORT_MAGIC, BFAD_VPORT_GET_STATS, struct bfa_ioctl_vport_stats_s)
#define IOCMD_VPORT_RESET_STATS \
	_IOWR(VPORT_MAGIC, BFAD_VPORT_RESET_STATS, \
	      struct bfa_ioctl_reset_stats_s)
#define IOCMD_VF_GET_STATS \
	_IOWR(VF_MAGIC, BFAD_VF_GET_STATS, struct bfa_ioctl_vf_stats_s)
#define IOCMD_VF_RESET_STATS \
	_IOWR(VF_MAGIC, BFAD_VF_RESET_STATS, struct bfa_ioctl_vf_reset_stats_s)
#define IOCMD_QOS_ENABLE \
	_IOWR(QOS_MAGIC, BFAD_QOS_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_QOS_DISABLE \
	_IOWR(QOS_MAGIC, BFAD_QOS_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_QOS_GET_ATTR \
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_ATTR, struct bfa_ioctl_qos_attr_s)
#define IOCMD_QOS_GET_VC_ATTR \
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_VC_ATTR, struct bfa_ioctl_qos_vc_attr_s)
#define IOCMD_QOS_GET_STATS \
	_IOWR(QOS_MAGIC, BFAD_QOS_GET_STATS, struct bfa_ioctl_qos_vc_attr_s)
#define IOCMD_QOS_RESET_STATS \
	_IOWR(QOS_MAGIC, BFAD_QOS_RESET_STATS, struct bfa_ioctl_gen_s)
#define IOCMD_RATELIM_ENABLE \
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_RATELIM_DISABLE \
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_RATELIM_DEF_SPEED \
	_IOWR(RATELIM_MAGIC, BFAD_RATELIM_DEF_SPEED, \
	      struct bfa_ioctl_trl_speed_s)
#define IOCMD_FCPIM_FAILOVER \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_FAILOVER, struct bfa_ioctl_fcpim_s)
#define IOCMD_FCPIM_MODSTATS \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_MODSTATS, \
	      struct bfa_ioctl_fcpim_modstats_s)
#define IOCMD_FCPIM_MODSTATSCLR \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_MODSTATSCLR, \
	      struct bfa_ioctl_fcpim_modstatsclr_s)
#define IOCMD_FCPIM_PROFILE_ON \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_PROFILE_ON, \
	      struct bfa_ioctl_fcpim_profile_s)
#define IOCMD_FCPIM_PROFILE_OFF \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_PROFILE_OFF, \
	      struct bfa_ioctl_fcpim_profile_s)
#define IOCMD_FCPIM_DEL_ITN_STATS \
	_IOWR(FCPIM_MAGIC, BFAD_FCPIM_DEL_ITN_STATS, \
	      struct bfa_ioctl_fcpim_del_itn_stats_s)
#define IOCMD_BOOT_CFG \
	_IOWR(BOOT_MAGIC, BFAD_BOOT_CFG, struct bfa_ioctl_boot_s)
#define IOCMD_BOOT_QUERY \
	_IOWR(BOOT_MAGIC, BFAD_BOOT_QUERY, struct bfa_ioctl_boot_s)
#define IOCMD_PREBOOT_QUERY \
	_IOWR(BOOT_MAGIC, BFAD_PREBOOT_QUERY, struct bfa_ioctl_preboot_s)
#define IOCMD_ITNIM_GET_ATTR \
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_ATTR, struct bfa_ioctl_itnim_attr_s)
#define IOCMD_ITNIM_GET_IOSTATS \
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_IOSTATS, \
	      struct bfa_ioctl_itnim_iostats_s)
#define IOCMD_ITNIM_RESET_STATS \
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_RESET_STATS, \
	      struct bfa_ioctl_rport_reset_stats_s)
#define IOCMD_ITNIM_GET_ITNSTATS \
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_ITNSTATS, \
	      struct bfa_ioctl_itnim_itnstats_s)
#define IOCMD_ITNIM_GET_IOPROFILE \
	_IOWR(ITNIM_MAGIC, BFAD_ITNIM_GET_IOPROFILE, \
	      struct bfa_ioctl_itnim_ioprofile_s)
#define IOCMD_FABRIC_GET_LPORTS \
	_IOWR(FABRIC_MAGIC, BFAD_FABRIC_GET_LPORTS, \
	      struct bfa_ioctl_fabric_get_lports_s)
#define IOCMD_FCPORT_GET_STATS \
	_IOWR(FCPORT_MAGIC, BFAD_FCPORT_GET_STATS, \
	      struct bfa_ioctl_fcport_stats_s)
#define IOCMD_FCPORT_RESET_STATS \
	_IOWR(FCPORT_MAGIC, BFAD_FCPORT_RESET_STATS, struct bfa_ioctl_gen_s)
#define IOCMD_FCPORT_ENABLE \
	_IOWR(FCPORT_MAGIC, BFAD_FCPORT_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_FCPORT_DISABLE \
	_IOWR(FCPORT_MAGIC, BFAD_FCPORT_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_TRUNK_ENABLE \
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_ENABLE, struct bfa_ioctl_gen_s)
#define IOCMD_TRUNK_DISABLE \
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_DISABLE, struct bfa_ioctl_gen_s)
#define IOCMD_TRUNK_GET_ATTR \
	_IOWR(TRUNK_MAGIC, BFAD_TRUNK_GET_ATTR, struct bfa_ioctl_trunk_attr_s)
#define IOCMD_FAA_QUERY \
	_IOWR(FAA_MAGIC, BFAD_FAA_QUERY, struct bfa_ioctl_faa_attr_s)
#define IOCMD_SEND_FC           \
	_IOWR(PASSTHRU_MAGIC, BFAD_SEND_FC, bfa_ioctl_fcpt_t)

#endif /* BFAD_IOCTL_H */
